"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigAutoStore = void 0;
const lodash = require("lodash");
const ConfigUtils = require("./ConfigUtils");
const AbstractAuthHandler_1 = require("../../imperative/src/auth/handlers/AbstractAuthHandler");
const utilities_1 = require("../../utilities");
const Session_1 = require("../../rest/src/session/Session");
const SessConstants_1 = require("../../rest/src/session/SessConstants");
const logger_1 = require("../../logger");
/**
 * Class to manage automatic storage of properties in team config.
 */
class ConfigAutoStore {
    /**
     * Finds the profile where auto-store properties should be saved.
     * @param params CLI handler parameters object
     * @param profileProps List of properties required in the profile schema
     * @returns Tuple containing profile type and name, or undefined if no
     *          profile was found
     */
    static findActiveProfile(params, profileProps) {
        var _a, _b;
        const profileTypes = [
            ...(((_a = params.definition.profile) === null || _a === void 0 ? void 0 : _a.required) || []),
            ...(((_b = params.definition.profile) === null || _b === void 0 ? void 0 : _b.optional) || [])
        ];
        for (const profType of profileTypes) {
            const profileMatch = utilities_1.ImperativeConfig.instance.loadedConfig.profiles.find(p => p.type === profType);
            if (profileMatch != null && profileProps.every(propName => propName in profileMatch.schema.properties)) {
                return [profType, ConfigUtils.getActiveProfileName(profType, params.arguments)];
            }
        }
    }
    /**
     * Finds the token auth handler class for a team config profile.
     * @param profilePath JSON path of profile
     * @param cmdArguments CLI arguments which may specify a profile
     * @returns Auth handler class or undefined if none was found
     */
    static findAuthHandlerForProfile(profilePath, cmdArguments) {
        const config = utilities_1.ImperativeConfig.instance.config;
        const profileType = lodash.get(config.properties, `${profilePath}.type`);
        const profile = config.api.profiles.get(profilePath.replace(/profiles\./g, ""));
        if (profile == null || profileType == null) { // Profile must exist and have type defined
            return;
        }
        else if (profileType === "base") {
            if (profile.tokenType == null) { // Base profile must have tokenType defined
                return;
            }
        }
        else {
            if (profile.basePath == null) { // Service profiles must have basePath defined
                return;
            }
            else if (profile.tokenType == null) { // If tokenType undefined in service profile, fall back to base profile
                const baseProfileName = ConfigUtils.getActiveProfileName("base", cmdArguments);
                return this.findAuthHandlerForProfile(config.api.profiles.expandPath(baseProfileName), cmdArguments);
            }
        }
        const authConfigs = [];
        utilities_1.ImperativeConfig.instance.loadedConfig.profiles.forEach((profCfg) => {
            if ((profCfg.type === profileType || profCfg.type === "base") && profCfg.authConfig != null) {
                authConfigs.push(...profCfg.authConfig);
            }
        });
        for (const authConfig of authConfigs) {
            const authHandler = require(authConfig.handler);
            const authHandlerClass = new authHandler.default();
            if (authHandlerClass instanceof AbstractAuthHandler_1.AbstractAuthHandler) {
                const promptParams = authHandlerClass.getPromptParams()[0];
                if (profile.tokenType === promptParams.defaultTokenType) {
                    return authHandlerClass; // Auth service must have matching token type
                }
            }
        }
    }
    /**
     * Stores session config properties into a team config profile.
     * @param params CLI handler parameters object
     * @param sessCfg Session config containing properties to store
     * @param propsToStore Names of properties that should be stored
     */
    static storeSessCfgProps(params, sessCfg, propsToStore) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = utilities_1.ImperativeConfig.instance.config;
            // TODO Which autoStore value should take priority if it conflicts between layers
            if (propsToStore.length == 0 || !(config === null || config === void 0 ? void 0 : config.exists) || !config.properties.autoStore) {
                return;
            }
            let profileProps = propsToStore.map(propName => propName === "hostname" ? "host" : propName);
            const profileData = this.findActiveProfile(params, profileProps);
            if (profileData == null) {
                return;
            }
            const [profileType, profileName] = profileData;
            const profilePath = config.api.profiles.expandPath(profileName);
            // Replace user and password with tokenValue if tokenType is defined in config
            if (profileProps.includes("user") && profileProps.includes("password") &&
                (yield this.fetchTokenForSessCfg(params, sessCfg, profilePath))) {
                profileProps = profileProps.filter(propName => propName !== "user" && propName !== "password");
                profileProps.push("tokenValue");
            }
            const beforeLayer = config.api.layers.get();
            if (config.api.profiles.exists(profileName)) {
                const { user, global } = config.api.layers.find(profileName);
                config.api.layers.activate(user, global);
            }
            const profileObj = config.api.profiles.get(profileName);
            const profileSchema = utilities_1.ImperativeConfig.instance.loadedConfig.profiles.find(p => p.type === profileType).schema;
            const profileSecureProps = config.api.secure.securePropsForProfile(profileName);
            const baseProfileName = ConfigUtils.getActiveProfileName("base", params.arguments);
            const baseProfileObj = config.api.profiles.get(baseProfileName);
            const baseProfileSchema = utilities_1.ImperativeConfig.instance.loadedConfig.baseProfile.schema;
            const baseProfileSecureProps = config.api.secure.securePropsForProfile(baseProfileName);
            for (const propName of profileProps) {
                let propProfilePath = profilePath;
                /* If any of the following is true, then property should be stored in base profile:
                    (1) Service profile does not exist, but base profile does
                    (2) Property is missing from service profile properties/secure objects, but present in base profile
                    (3) Property is tokenValue and tokenType is missing from service profile, but present in base profile
                */
                if ((!config.api.profiles.exists(profileName) && config.api.profiles.exists(baseProfileName)) ||
                    (profileObj[propName] == null && !profileSecureProps.includes(propName) &&
                        (baseProfileObj[propName] != null || baseProfileSecureProps.includes(propName))) ||
                    (propName === "tokenValue" && profileObj.tokenType == null && baseProfileObj.tokenType != null)) {
                    propProfilePath = config.api.profiles.expandPath(baseProfileName);
                }
                const sessCfgPropName = propName === "host" ? "hostname" : propName;
                const propDefinition = profileSchema.properties[propName] || baseProfileSchema.properties[propName];
                // If secure array at higher level includes this property, then property should be stored at higher level
                if (propDefinition.secure) {
                    const secureProfilePath = config.api.secure.secureInfoForProp(`${propProfilePath}.properties.${propName}`, true).path;
                    if (secureProfilePath.split(".").length < propProfilePath.split(".").length) {
                        propProfilePath = secureProfilePath.substr(0, secureProfilePath.lastIndexOf("."));
                    }
                }
                config.set(`${propProfilePath}.properties.${propName}`, sessCfg[sessCfgPropName], {
                    secure: propDefinition.secure
                });
            }
            yield config.save(false);
            params.response.console.log(`Stored properties in ${config.layerActive().path}: ${profileProps.join(", ")}`);
            // Restore original active layer
            config.api.layers.activate(beforeLayer.user, beforeLayer.global);
        });
    }
    /**
     * Retrieves token value that will be auto-stored into session config.
     * @param params CLI handler parameters object
     * @param sessCfg Session config with credentials for basic or cert auth
     * @param profilePath JSON path of profile containing tokenType
     * @returns True if auth handler was found and token was fetched
     */
    static fetchTokenForSessCfg(params, sessCfg, profilePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const authHandlerClass = this.findAuthHandlerForProfile(profilePath, params.arguments);
            if (authHandlerClass == null) {
                return false;
            }
            const [promptParams, loginHandler] = authHandlerClass.getPromptParams();
            sessCfg.type = SessConstants_1.AUTH_TYPE_TOKEN;
            sessCfg.tokenType = promptParams.defaultTokenType;
            const baseSessCfg = { type: sessCfg.type };
            for (const propName of Object.keys(utilities_1.ImperativeConfig.instance.loadedConfig.baseProfile.schema.properties)) {
                const sessCfgPropName = propName === "host" ? "hostname" : propName;
                if (sessCfg[sessCfgPropName] != null) {
                    baseSessCfg[sessCfgPropName] = sessCfg[sessCfgPropName];
                }
            }
            logger_1.Logger.getAppLogger().info(`Fetching ${sessCfg.tokenType} for ${profilePath}`);
            sessCfg.tokenValue = yield loginHandler(new Session_1.Session(baseSessCfg));
            return true;
        });
    }
}
exports.ConfigAutoStore = ConfigAutoStore;
//# sourceMappingURL=ConfigAutoStore.js.map